/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

import minijava.node.*;
import minijava.analysis.*;
import java.util.HashMap;

class ClassFinder extends DepthFirstAdapter {

    HashMap classes;
    SymbolTable functions;

    public ClassFinder() {
	classes = new HashMap();
	functions = new SymbolTable();
    }

    HashMap getClasses() {
	return classes;
    }

    SymbolTable getFunctions() {
	return functions;
    }

    public void caseAClassDecDeclaration( AClassDecDeclaration node ) {

	if ( node.getIdentifier() != null ) {
	    String classname = node.getIdentifier().toString().trim();
	    classes.put( classname , null );	
	}
    }    


    public void caseAFuncDecDeclaration( AFuncDecDeclaration node ) {

	if ( node.getIdentifier() != null ) {

	    String name = node.getIdentifier().toString().trim();
	    String type = node.getType().toString().trim();
	    
	    if ( !functions.containsKey( name ) ) {
		SymbolTable entry = new SymbolTable( type );
		entry.prototypeOnly = true;
		functions.put( name , entry );
	    }
	}
    }
}
