/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

class ExecuteListener implements ActionListener
{

    private KenyaUI ui;
    private Process theProcess = null;
    private BufferedReader inStream = null;
    private BufferedReader errStream = null;
    private String slash, pathsep, java, temp, gj, kenya, compiler;

    ExecuteListener( KenyaUI f )
    {
	ui = f;

	slash = System.getProperty("file.separator");
	pathsep = System.getProperty("path.separator");
	java = ui.prop.getProperty("java.location");
	kenya = ui.prop.getProperty("kenya.location");
	gj = ui.prop.getProperty("gj.location");
	compiler = ui.prop.getProperty("compiler");
        temp = ui.prop.getProperty("working.dir");
    }

    public void actionPerformed( ActionEvent event )
    {

	if ( ui.lastCompiledKCode.equals("") ) {
	    ui.msgsArea.setText("No code has been compiled.");
	    return;
	}

	if ( ! ui.kenyaArea.getText().trim().equals( ui.lastParsedKCode ) ) {
	    ui.msgsArea.setText("Warning: Kenya code altered since last compile. Running old version.");

	    //possibly ask user whether to proceed or recompile etc... ??

	} else {
	    
	    ui.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
	    ui.msgsArea.setText("Running ...");
	    ui.out = new OutputWindow(ui);
	    ui.out.outputArea.setEditable(false);
	}
	
	//SwingWorker object executes its methods in a separate thread

	final SwingWorker worker = new SwingWorker() {

		public Object construct() {	  

		    try {
			
			String cmd;
			
			if ( compiler.equals("gj") ) {
			    cmd  = java + " -cp " + SaveListener.getSavePath() + pathsep + temp + pathsep + gj + slash + "classes" + pathsep + kenya + " " + ui.packagename;
			} else {
			    
			    //compiler == javac
			    cmd  = java + " -cp " + SaveListener.getSavePath() + pathsep + temp + pathsep + kenya + " " + ui.packagename;
			}

			theProcess = Runtime.getRuntime().exec(cmd);

			if ( theProcess == null ) {
			    System.err.println( "Unable to spawn external process, error executing command: " + cmd );
			    return new Integer(-1);
			}

			inStream = new BufferedReader( new InputStreamReader( theProcess.getInputStream() ));
			errStream = new BufferedReader( new InputStreamReader( theProcess.getErrorStream() ));
		    }
		    
		    catch(IOException e) {
			System.err.println("Error spawning external process");
			e.printStackTrace();  
		    }
		    
		    ui.out.connectProcess( theProcess );
		    ui.out.pack();
		    ui.out.setVisible(true);

		    int c;
		    ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );

		    try	{

			while ( ( ( c = inStream.read() ) != -1 ) || ( ( c = errStream.read() ) != -1 ) ) { 
			    
			    ui.out.outputArea.append( String.valueOf( (char)c ) );  //line.toString() );
			    ui.out.outputArea.setCaretPosition( ui.out.outputArea.getText().length() );
			}	
			
		
  
			inStream.close();
			
			if ( ui.out.inputbox.o != null ) {
			    ui.out.inputbox.o.close();
			}
		    }

		    catch(IOException e) {
			System.err.println("Error on inStream.readLine()");
			e.printStackTrace();  
		    }
		    
		    ui.msgsArea.append(" Finished.");
		    
		    return new Integer(0);
		}
	    };
	
	//activate the code in the SwingWorker object
	
	worker.start();
    }
}
