/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreeSelectionModel;
import java.net.URL;
import java.io.IOException;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;

class HelpWindow extends JFrame {

    private KenyaUI ui;
    private JEditorPane htmlPane;
    private static boolean DEBUG = false;
    private URL helpURL;

    private boolean playWithLineStyle = false;
    private String lineStyle = "Angled"; 
    String slash;

    HelpWindow( KenyaUI u ) {

        super("Kenya Help");
	ui = u;
	slash = System.getProperty("file.separator");

        //Create the nodes.
        DefaultMutableTreeNode top = new DefaultMutableTreeNode("Kenya");
        createNodes(top);

        //Create a tree that allows one selection at a time.
        final JTree tree = new JTree(top);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

        //Listen for when the selection changes.
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();

                if (node == null) return;

                Object nodeInfo = node.getUserObject();
                if (node.isLeaf()) {
                    BookInfo book = (BookInfo)nodeInfo;
                    displayURL(book.bookURL);
                    if (DEBUG) {
                        System.out.print(book.bookURL + ":  \n    ");
                    }
                } else {
                    displayURL(helpURL); 
                }
                if (DEBUG) {
                    System.out.println(nodeInfo.toString());
                }
            }
        });

        if (playWithLineStyle) {
            tree.putClientProperty("JTree.lineStyle", lineStyle);
        }


        JScrollPane treeView = new JScrollPane(tree);
        htmlPane = new JEditorPane();
        htmlPane.setEditable(false);
        initHelp();
        JScrollPane htmlView = new JScrollPane(htmlPane);

        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setTopComponent(treeView);
        splitPane.setBottomComponent(htmlView);

        Dimension minimumSize = new Dimension(100, 50);
        htmlView.setMinimumSize(minimumSize);
        treeView.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(100); 
        splitPane.setPreferredSize(new Dimension(500, 300));
        getContentPane().add(splitPane, BorderLayout.CENTER);
    }

    private class BookInfo {

        public String bookName;
        public URL bookURL;
        public String prefix = "file:" + ui.prop.getProperty("kenya.help") + slash + "help" + slash;

        public BookInfo(String book, String filename) {
            bookName = book;
            try {
                bookURL = new URL(prefix + filename);
            } catch (java.net.MalformedURLException exc) {
                System.err.println("Attempted to create a BookInfo "
                                   + "with a bad URL: " + bookURL);
                bookURL = null;
            }
        }

        public String toString() {
            return bookName;
        }
    }

    private void initHelp() {
        String s = null;
        try {
            s = "file:" + ui.prop.getProperty("kenya.help") +  slash + "help" + slash + "HelpIntro.html";
            if (DEBUG) {
                System.out.println("Help URL is " + s);
            }
            helpURL = new URL(s);
            displayURL(helpURL);
        } catch (Exception e) {
            System.err.println("Couldn't create help URL: " + s);
        }
    }

    private void displayURL(URL url) {

        try {
            htmlPane.setPage(url);
        } catch (IOException e) {
            System.err.println("Attempted to read a bad URL: " + url);
        }
    }

    private void createNodes(DefaultMutableTreeNode top) {

        DefaultMutableTreeNode category = null;
        DefaultMutableTreeNode book = null;

        category = new DefaultMutableTreeNode("Kenya language reference");
        top.add(category);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Types",
            "types.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Declaring Variables",
            "declaring.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Using Arrays",
            "arrays.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Writing Expressions",
            "expressions.html"));
        category.add(book); 

	book = new DefaultMutableTreeNode(new BookInfo
            ("Using Assertions",
            "assert.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Text Output",
            "output.html"));
        category.add(book);

	book = new DefaultMutableTreeNode(new BookInfo
	    ("Graphical Output",
	     "graphics.html"));
	category.add(book);

	book = new DefaultMutableTreeNode(new BookInfo
            ("User Input",
            "input.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Loops",
            "loops.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Conditional Statements",
            "if.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Switch",
	     "switch.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Using Functions",
            "funcs.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("User Defined Data Types",
            "udts.html"));
        category.add(book);

        category = new DefaultMutableTreeNode("Using the Kenya system");
        top.add(category);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Writing a Program",
             "writing.html"));
        category.add(book);

        book = new DefaultMutableTreeNode(new BookInfo
            ("Running a Program",
             "running.html"));
        category.add(book); 

	book = new DefaultMutableTreeNode(new BookInfo
            ("Providing a Program with Input",
             "userinput.html"));
        category.add(book);
	
        book = new DefaultMutableTreeNode(new BookInfo
            ("Accessing the help",
             "help.html"));
        category.add(book);
    }
}
