/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

import java.util.*;
import java.awt.event.*;
import java.awt.Cursor;
import java.io.PushbackReader;
import java.io.StringReader;

import javax.swing.*;

import minijava.node.*;
import minijava.lexer.*;
import minijava.parser.*;

class ParseListener implements ActionListener
{
    static Vector funcs;
    static Vector classes;
    static boolean tree;
    private static KenyaUI ui; 

    ParseListener(KenyaUI f)
    {
	ui = f;
    }

    public void actionPerformed( ActionEvent event )
    {
	parse();
    }

    static void parse() 
    {

	if(  ui.kenyaArea.getText().trim().equals("") ) {
	    ui.msgsArea.setText("No Kenya to parse.");
	    return;
	} 
	// cla97 05/10/2001 - Fix for Save As >>
	/*if ( ui.kenyaArea.getText().trim().equals( ui.lastParsedKCode ) ) {
	    ui.msgsArea.setText("Kenya unchanged since last parse.");
	    return;
	}*/
	// cla97 05/10/2001 <<

	while ( ui.packagename.equals("") ) {
		// cla97 05/10/2001 >>
		/*
	    String p = JOptionPane.showInputDialog( ui , "Please enter a filename:");
	    if ( p == null ) { return; }
	    if ( p.equals( "") || ! Util.isValidIdentifier( p ) ) { 
		JOptionPane.showMessageDialog( ui , "Invalid filename (only letters are allowed)" );
		continue;
	    } 
	    ui.packagename = p;  
	    */
	    SaveListener.save(ui,true);
	    // cla97 05/10/2001 <<
	}

	ui.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
	ui.lastParsedKCode = ui.kenyaArea.getText().trim();
	
	try {
	    ui.msgsArea.setText("");
	    ui.javaArea.setText("");
	    
	    Lexer lexer = new Lexer( new PushbackReader( new StringReader( ui.kenyaArea.getText() )));	
	    Parser parser = new Parser(lexer);
	    Start ast = parser.parse();
	    
	    funcs = new Vector();
	    classes = new Vector();
	    tree = true;
	    ui.highlight.removeAllHighlights();
	    
	    // Typecheck
	    
	    ClassFinder cf = new ClassFinder();
	    
	    ast.apply(cf);

	    TypeChecker tc = new TypeChecker( ui , cf.getClasses() , cf.getFunctions() );
	    tc.functions = cf.getFunctions();
	    ast.apply(tc);
	    
	    if ( tc.genCode ) {
		
		Translator t = new Translator(tc);
		    
		//Translate

		ast.apply(t);
		    
		tree = false;
		    
		for (Enumeration e = funcs.elements() ; e.hasMoreElements() ;) {
		    t.addLine("\n");
		    Node n = (Node)e.nextElement();
		    n.apply(t);
		}
		    
		t.addLine("\n}\n");
		
		for (Enumeration e = classes.elements() ; e.hasMoreElements() ;) {
		    Node n = (Node)e.nextElement();
		    n.apply(t);
		    t.addLine("\n");
		}
		
		Vector v = t.getCode();
		Iterator i = t.getImports();
		
		//cla97 05/10/2001 >>
		//ui.javaArea.setText("package " + ui.packagename + ";\n\n");
		//cla97 05/10/2001 <<
		if (  i.hasNext() ) {

		    if ( ! ( ui.prop.getProperty("compiler").equals("gj") ) ) {
			ui.msgsArea.append("Warning: Your compiler (" + ui.prop.getProperty("compiler") + 
					   ") does not support parameterised types.\n");
			}
			    
			while ( i.hasNext() ) {
			    ui.javaArea.append( "import gj.util." + ((String)i.next()).trim() + ";\n" );
			}
			
			ui.javaArea.append( "\n" );
		}

		if ( t.usingGraphics )
		    ui.javaArea.append( "import kenya.GraphicsWindow;\n\n");
		
		if ( t.getReads() ) {
		    ui.javaArea.append( "import kenya.io.InputReader;\n\n" );
		}
		//cla97 05/10/2001 >>
		ui.javaArea.append("public class "+ui.packagename +" {\n");
		//cla97 05/10/2001 <<
		Iterator constit  = t.getConstants();
		
		while ( constit.hasNext() ) {
		    Translator forconsts = new Translator();
		    ((Node)constit.next()).apply( forconsts );
		    for (Enumeration e = forconsts.getCode().elements() ; e.hasMoreElements() ; ) {
			CodeLine c = (CodeLine)e.nextElement();
			ui.javaArea.append( c.code );
		    }
		}	        

		for (Enumeration e = v.elements() ; e.hasMoreElements() ; ) {
		    CodeLine c = (CodeLine)e.nextElement();
		    
		    if ( c.newline ) {
			ui.javaArea.append("\n");
			
			if ( ! e.hasMoreElements() ) {
			    break;
			}
			    
			CodeLine next = (CodeLine)e.nextElement();

			for ( int j = 0 ; j < next.indent ; j++ ) 
			    ui.javaArea.append("\t");

			if ( next.code.equals("graphics") ) {

			    if ( t.usingGraphics ) {
				ui.javaArea.append("GraphicsWindow gw = new GraphicsWindow();\n\n");
				for ( int j = 0 ; j < next.indent ; j++ ) 
				    ui.javaArea.append("\t");
			    }
				
			} else {
			    
			    ui.javaArea.append( next.code );
			}
		    }
		    
		    ui.javaArea.append( c.code );
		}
	    }		
	    ui.msgsArea.append("Finished parsing."); 
	    ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
	}	
	
	catch ( minijava.parser.ParserException e ) {

	    ui.msgsArea.append( e.getMessage() );
	    
	    try {
		
		java.awt.Point start = new java.awt.Point( 0 , e.getToken().getLine() * ui.fontheight );
		java.awt.Point end = new java.awt.Point( ui.kenyaArea.getWidth() , e.getToken().getLine() * ui.fontheight  );
		ui.highlight.addHighlight( ui.kenyaArea.viewToModel( start ) , ui.kenyaArea.viewToModel( end ) , ui.painter );
		ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
		
	    } catch ( javax.swing.text.BadLocationException ble ) { System.err.println(ble); }
	} 
	
	catch ( minijava.lexer.LexerException e ) {
	    ui.msgsArea.append( e.getMessage() );

	    try {
		
		java.awt.Point start = new java.awt.Point( 0 , e.line * ui.fontheight );
		java.awt.Point end = new java.awt.Point( ui.kenyaArea.getWidth() , e.line * ui.fontheight  );
		ui.highlight.addHighlight( ui.kenyaArea.viewToModel( start ) , ui.kenyaArea.viewToModel( end ) , ui.painter );
		ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
		
	    } catch ( javax.swing.text.BadLocationException ble ) { System.err.println(ble); }

	}

	catch ( java.io.IOException e ) { ui.msgsArea.append("Error parsing program. Probable syntax error."); }

	catch ( Exception e ) {
	    System.err.println( e );
	}
    }
}
